@@ -1,20 +1,32 @@
+#![allow(unused)]
 use bromine::context::Context;
 use bromine::protocol::AsyncStreamProtocolListener;
 use bromine::IPCBuilder;
 use call_counter::*;
 use lazy_static::lazy_static;
-use std::sync::atomic::{AtomicU8, Ordering};
+use std::sync::atomic::{AtomicBool, AtomicU8, Ordering};
 use std::sync::Arc;
 use tokio::sync::oneshot::channel;
 
 pub mod call_counter;
 pub mod protocol;
 
+pub fn setup() {
+    lazy_static! {
+        static ref SETUP_DONE: Arc<AtomicBool> = Default::default();
+    }
+    if !SETUP_DONE.swap(true, Ordering::SeqCst) {
+        tracing_subscriber::fmt::init();
+    }
+}
+
 pub fn get_free_port() -> u8 {
     lazy_static! {
         static ref PORT_COUNTER: Arc<AtomicU8> = Arc::new(AtomicU8::new(0));
     }
-    PORT_COUNTER.fetch_add(1, Ordering::Relaxed)
+    let count = PORT_COUNTER.fetch_add(1, Ordering::Relaxed);
+
+    count
 }
 
 pub async fn start_server_and_client<
@@ -23,6 +35,7 @@ pub async fn start_server_and_client<
 >(
     builder_fn: F,
 ) -> Context {
+    setup();
     let counters = CallCounter::default();
     let (sender, receiver) = channel::<()>();
     let client_builder = builder_fn().insert::<CallCounterKey>(counters.clone());
